@@ -5,10 +5,11 @@ use Net::FTP;
 use Cwd;
 use File::Path;
 use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs );
+use Test::Smoke::Util qw( clean_filename );
 
-# $Id: FTPClient.pm 923 2006-01-20 16:13:32Z abeltje $
-use vars qw( $VERSION $NOCASE );
-$VERSION = '0.005';
+# $Id: FTPClient.pm 1042 2007-04-06 21:18:34Z abeltje $
+use vars qw( $VERSION );
+$VERSION = '0.007';
 
 my %CONFIG = (
     df_fserver  => undef,
@@ -22,9 +23,7 @@ my %CONFIG = (
 );
 my @sn = qw( B KB MB GB TB );
 
-$NOCASE = $^O eq 'MSWin32' || $^O eq 'VMS';
-
-BEGIN { eval qq[use Time::HiRes qw( time ) ] }
+BEGIN { eval qq/use Time::HiRes qw( time ) / }
 
 =head1 NAME
 
@@ -154,7 +153,7 @@ sub mirror {
 
     if ( $self->{ftype} && $self->{client}->can( $self->{ftype} ) ) {
         my $ftype = $self->{ftype};
-        $self->{client}->$ftype;
+        eval '$self->{client}->$ftype';
     }
     my( $totsize, $tottime ) = ( 0, 0 );
     $self->{v} and print "Start mirror to: $ddir\n";
@@ -252,7 +251,7 @@ sub __do_mirror {
             $verbose > 1 and print "Leaving '$entry->{name}' [$new_locald]\n";
         } else {
             $entry->{time}  = $ftp->mdtm( $entry->{name} ); #slow down
-            my $fname = __clean_name( $entry->{name} );
+            my $fname = clean_filename( $entry->{name} );
 
             my $destname = catfile( $localdir, canonpath($fname) );
 
@@ -296,12 +295,12 @@ sub __do_mirror {
     if ( $cleanup ) {
         $verbose > 1 and print "Cleanup '$localdir'\n";
         my %ok_file = map {
-            ( __clean_name( $_->{name} ) => $_->{type} )
+            ( clean_filename( $_->{name} ) => $_->{type} )
         } @list;
         local *DIR;
         if ( opendir DIR, '.' ) {
             foreach ( readdir DIR ) {
-                my $cmpname = __clean_name( $_ );
+                my $cmpname = clean_filename( $_ );
                 $^O eq 'VMS' and $cmpname =~ s/\.$//;
                 if( -f $cmpname ) {
                     unless ( exists $ok_file{ $cmpname } && 
@@ -327,19 +326,6 @@ sub __do_mirror {
 }
 }
 
-sub __clean_name {
-    my $fname = shift;
-
-    if ( $^O eq 'VMS' ) {
-        my @parts = split /[.@#]/, $fname;
-       	if ( @parts > 1 ) {
-            my $ext = ( pop @parts ) || '';
-            $fname = join( "_", @parts ) . ".$ext";
-        }
-    }
-    return $NOCASE ? uc $fname : $fname;
-}
-
 =head2 dirlist( $ftp, $verbose )
 
 Return a list of entries (hashrefs) with these properties:
