@@ -1,9 +1,9 @@
 package Test::Smoke::Reporter;
 use strict;
 
-# $Id: Reporter.pm 894 2005-07-31 15:44:42Z abeltje $
+# $Id: Reporter.pm 1076 2007-08-30 13:02:23Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.026';
+$VERSION = '0.027';
 
 use Cwd;
 use File::Spec::Functions;
@@ -173,7 +173,7 @@ sub _read {
                 $vmsg = "from $nameorref";
             } else {
                 require Carp;
-                Carp::carp "Cannot read smokeresults ($nameorref): $!";
+                Carp::carp( "Cannot read smokeresults ($nameorref): $!" );
                 $self->{_outfile} = undef;
                 $vmsg = "did fail";
             }
@@ -363,7 +363,7 @@ sub _post_process {
                 next if $tstenv eq 'minitest' && ! exists $status->{ $tstenv };
 
                 ( my $showenv = $tstenv ) =~ s/^locale://;
-                $self->{_locale} ||= $showenv if $tstenv =~ /^locale:/;
+                $tstenv =~ /^locale:/ and push @{ $self->{_locale} }, $showenv;
                 $showenv = 'default' 
                     if $self->{defaultenv} && $showenv eq 'stdio';
 
@@ -390,9 +390,11 @@ sub _post_process {
             }
             unless ( $self->{defaultenv} ) {
                 exists $status->{perlio} or $status->{perlio} = '-';
-                exists $status->{ "locale:$self->{locale}" } or 
-                    $status->{ "locale:$self->{locale}" } = '-'
-                        if $self->{locale};
+                my @locales = split ' ', ($self->{locale} || '');
+                for my $locale ( @locales ) {
+                    exists $status->{ "locale:$locale" } or 
+                        $status->{ "locale:$locale" } = '-'
+                }
             }
 
             $count{ $_ }++ for map {
@@ -400,6 +402,10 @@ sub _post_process {
             } map $status->{ $_ } => keys %$status;
         }
     }
+    defined $self->{_locale} or $self->{_locale} = [ ];
+    { local $, = $" = "', '"; 
+    $self->{v} and print "Found locale: '@{ $self->{_locale}}'\n";
+    }
     my @failures = map {
         { tests => $_,
           cfgs  => [ map {
@@ -457,13 +463,13 @@ sub write_to_file {
     local *RPT;
     open RPT, "> $name" or do {
         require Carp;
-        Carp::carp "Error creating '$name': $!";
+        Carp::carp( "Error creating '$name': $!" );
         return;
     };
     print RPT $self->report;
     close RPT or do {
         require Carp;
-        Carp::carp "Error writing to '$name': $!";
+        Carp::carp( "Error writing to '$name': $!" );
         return;
     };
     $self->{v} and print " OK\n";
@@ -704,11 +710,38 @@ Returns a string with the legend for build-environments
 
 sub bldenv_legend {
     my $self = shift;
-    my $locale = $self->{_locale};
     $self->{defaultenv} = ( @{ $self->{_tstenv} } == 1 )
         unless defined $self->{defaultenv};
     my $debugging = $self->{_rpt}{dbughow} || '-DDEBUGGING';
 
+    if ( $self->{_locale} && @{ $self->{_locale} } ) {
+        my $lcnt = @{ $self->{_locale} };
+        my $half = int(( 4 +  $lcnt ) / 2 );
+        my $cnt = 2 * $half;
+        my @locale = @{ $self->{_locale} };
+
+        my $line = '';
+        for my $i ( 0 .. $cnt-1 ) {
+            $line .= '| ' x ( $cnt - 1 - $i );
+            $line .= '+';
+            $line .= '-' x (2 * $i);
+            $line .= '- ';
+
+            if ( ($i % $half) < ($lcnt / 2) ) {
+                my $locale = shift @locale;     # XXX: perhaps pop()
+                $line .= "LC_ALL = $locale"
+            } else {
+                $line .= ( $i % 2 == 0 )
+                    ? "PERLIO = perlio"
+                    : "PERLIO = stdio";
+            }
+            $i < $half and $line .= " $debugging";
+            $line .= "\n";
+        }
+        return $line;
+    }
+
+    my $locale = ''; # XXX
     return  $locale ? <<EOL : $self->{defaultenv} ? <<EOS : <<EOE;
 | | | | | +- LC_ALL = $locale $debugging
 | | | | +--- PERLIO = perlio $debugging
