@@ -1,9 +1,9 @@
 package Test::Smoke::Syncer;
 use strict;
 
-# $Id: Syncer.pm 966 2006-05-25 19:07:16Z abeltje $
+# $Id: Syncer.pm 1072 2007-08-30 10:30:36Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.019';
+$VERSION = '0.023';
 
 use Config;
 use Cwd;
@@ -157,7 +157,7 @@ sub new {
 
     unless ( exists $CONFIG{valid_type}->{$sync_type} ) {
         require Carp;
-        Carp::croak "Invalid sync_type '$sync_type'";
+        Carp::croak( "Invalid sync_type '$sync_type'" );
     };
 
     my %args_raw = @_ ? UNIVERSAL::isa( $_[0], 'HASH' ) ? %{ $_[0] } : @_ : ();
@@ -187,7 +187,7 @@ sub new {
         /^forest$/   && return Test::Smoke::Syncer::Forest->new( %fields );
 
         require Carp;
-        Carp::croak "Synctype '$_', not yet implemented.";
+        Carp::croak( "Synctype '$_', not yet implemented." );
     }
 
 }
@@ -265,7 +265,8 @@ sub _relocate_tree {
     $self->{v} and print "relocate source-tree ";
 
     # try to move it at once (sort of a rename)
-    my $ok = $source_dir eq $self->{ddir}
+    my $ddir = $^O eq 'VMS' ? $self->{vms_ddir} : $self->{ddir};
+    my $ok = $source_dir eq $ddir
            ? 1 : File::Copy::move( $source_dir, $self->{ddir} );
 
     # Failing that: Copy-By-File :-(
@@ -426,7 +427,7 @@ sub pre_sync {
 # Set skeleton-sub
 sub sync { 
     require Carp; 
-    Carp::croak ref( $_[0] ) . "->sync() not yet implemented.";
+    Carp::croak( ref( $_[0] ) . "->sync() not yet implemented." );
 }
 
 =item $syncer->post_sync
@@ -510,7 +511,7 @@ sub sync {
     if ( system $command ) {
         my $err = $? >> 8;
         require Carp;
-        Carp::carp "Problem during rsync ($err)";
+        Carp::carp( "Problem during rsync ($err)" );
     }
 
     my $plevel = $self->check_dot_patch;
@@ -536,6 +537,7 @@ package Test::Smoke::Syncer::Snapshot;
 
 use Cwd;
 use File::Path;
+use Test::Smoke::Util qw( whereis clean_filename );
 
 =item Test::Smoke::Syncer::Snapshot->new( %args )
 
@@ -602,21 +604,21 @@ sub _fetch_snapshot {
     require Net::FTP;
     my $ftp = Net::FTP->new($self->{server}, Debug => 0, Passive => 1) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't open $self->{server}: $@";
+        Carp::carp( "[Net::FTP] Can't open $self->{server}: $@" );
         return undef;
     };
 
     my @login = ( $self->{ftpusr}, $self->{ftppwd} );
     $ftp->login( @login ) or do {
         require Carp;
-        Carp::carp "[Net:FTP] Can't login( @login )";
+        Carp::carp( "[Net:FTP] Can't login( @login )" );
         return undef;
     };
 
     $self->{v} and print "Connected to $self->{server}\n";
     $ftp->cwd( $self->{sdir} ) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't chdir '$self->{sdir}'";
+        Carp::carp( "[Net::FTP] Can't chdir '$self->{sdir}'" );
         return undef;
     };
 
@@ -625,7 +627,7 @@ sub _fetch_snapshot {
 
     unless ( $snap_name ) {
         require Carp;
-        Carp::carp "Couldn't find a snapshot at $self->{server}$self->{sdir}";
+        Carp::carp("Couldn't find a snapshot at $self->{server}$self->{sdir}");
         return undef;
     }
 
@@ -633,7 +635,8 @@ sub _fetch_snapshot {
     my $snap_size = $ftp->size( $snap_name );
     my $ddir_var = $self->{vms_ddir} ? 'vms_ddir' : 'ddir';
     my $local_snap = File::Spec->catfile( $self->{ $ddir_var },
-                                          File::Spec->updir, $snap_name );
+                                          File::Spec->updir,
+                                          clean_filename( $snap_name ) );
     $local_snap = File::Spec->canonpath( $local_snap );
 
     if ( -f $local_snap && $snap_size == -s $local_snap ) {
@@ -666,7 +669,7 @@ sub _fetch_snapshot_HTTP {
 
     unless ( $snap_name ) {
         require Carp;
-        Carp::carp "No snapshot specified for $self->{server}$self->{sdir}";
+        Carp::carp( "No snapshot specified for $self->{server}$self->{sdir}" );
         return undef;
     }
 
@@ -732,7 +735,7 @@ sub _extract_snapshot {
 
     unless ( $self->{snapshot} && -f $self->{snapshot} ) {
         require Carp;
-        Carp::carp "No snapshot to be extracted!";
+        Carp::carp( "No snapshot to be extracted!" );
         return undef;
     }
 
@@ -741,11 +744,11 @@ sub _extract_snapshot {
     # Files in the snapshot are relative to the 'perl/' directory,
     # they may need to be moved and that is not easy when you've
     # extracted them in the target directory! so we go updir()
-    my $extract_base = File::Spec->catdir( $self->{ddir},
-                                           File::Spec->updir );
+    my $ddir = $^O eq 'VMS' ? $self->{vms_ddir} : $self->{ddir};
+    my $extract_base = File::Spec->catdir( $ddir, File::Spec->updir );
     chdir $extract_base or do {
         require Carp;
-        Carp::croak "Can't chdir '$extract_base': $!";
+        Carp::croak( "Can't chdir '$extract_base': $!" );
     };
 
     my $snap_base;
@@ -765,7 +768,7 @@ sub _extract_snapshot {
 
     chdir $cwd or do {
         require Carp;
-        Carp::croak "Can't chdir($extract_base) back: $!";
+        Carp::croak( "Can't chdir($extract_base) back: $!" );
     };
 
     if ( $self->{cleanup} & 1 ) {
@@ -788,7 +791,7 @@ sub _extract_with_Archive_Tar {
 
     my $archive = Archive::Tar->new() or do {
         require Carp;
-        Carp::carp "Can't Archive::Tar->new: " . $Archive::Tar::error;
+        Carp::carp( "Can't Archive::Tar->new: " . $Archive::Tar::error );
         return undef;
     };
 
@@ -796,7 +799,7 @@ sub _extract_with_Archive_Tar {
     $archive->read( $self->{snapshot}, 1 );
     $Archive::Tar::error and do {
         require Carp;
-        Carp::carp "Error reading '$self->{snapshot}': ".$Archive::Tar::error;
+        Carp::carp("Error reading '$self->{snapshot}': ".$Archive::Tar::error);
         return undef;
     };
     my @files = $archive->list_files;
@@ -824,12 +827,12 @@ sub _extract_with_external {
     if ( $^O ne 'VMS' ) {
         my $command = sprintf $self->{tar}, $self->{snapshot};
         $command .= " $self->{snapshot}" if $command eq $self->{tar};
-    
+
         $self->{v} and print "$command ";
         if ( system $command ) {
             my $error = $? >> 8;
             require Carp;
-            Carp::carp "Error in command: $error";
+            Carp::carp( "Error in command: $error" );
             return undef;
         };
         $self->{v} and print "OK\n";
@@ -844,7 +847,8 @@ sub _extract_with_external {
         foreach @dirs_pre;
     # I'll pick the first one that has 'perl' in it
     my( $prefix ) = grep /\bperl/ || /perl\b/ => keys %dirs_post;
-    $prefix ||= File::Spec->abs2rel( $self->{ddir}, cwd() );
+    my $ddir = $^O eq 'VMS' ? $self->{vms_ddir} : $self->{ddir};
+    $prefix ||= File::Spec->abs2rel( $ddir, cwd() );
 
     my $base_dir = File::Spec->canonpath(File::Spec->catdir( cwd(), $prefix ));
     $self->{v} and print "Snapshot prefix: '$base_dir'\n";
@@ -860,8 +864,8 @@ Gunzip and extract the archive in C<$tgzfile> using a small DCL script
 sub __vms_untargz {
     my( $cmd, $file, $verbose ) = @_;
     my( $gzip_cmd, $tar_cmd ) = split /\s*\|\s*/, $cmd;
-    my $gzip = $gzip_cmd =~ /^(\S+)/ ? $1 : 'GZIP';
-    my $tar  = $tar_cmd  =~ /^(\S+)/
+    my $gzip = $gzip_cmd =~ /^((?:MCR )?\S+)/ ? $1 : 'GZIP';
+    my $tar  = $tar_cmd  =~ /^((?:MCR )?\S+)/
         ? $1 : (whereis( 'vmstar' ) || whereis( 'tar' ) );
     my $tar_sw = $verbose ? '-xvf' : '-xf';
 
@@ -869,15 +873,16 @@ sub __vms_untargz {
     local *TMPCOM;
     open TMPCOM, "> TS-UNTGZ.COM" or return 0;
     print TMPCOM <<EO_UNTGZ; close TMPCOM or return 0;
-\$ define/user sys\$output TS-UNTGZ.TAR
-\$ $gzip "-cd" $file
-\$ $tar $tar_sw TS-UNTGZ.TAR
-\$ delete TS-UNTGZ.TAR;*
+\$! TS-UNTGZ.COM - Generated by Test::Smoke::Syncer
+\$  define/user sys\$output TS-UNTGZ.TAR
+\$  $gzip "-cd" $file
+\$  $tar $tar_sw TS-UNTGZ.TAR
+\$  delete TS-UNTGZ.TAR;*
 EO_UNTGZ
     $verbose and print " OK\n";
 
     my $ret = system "\@TS-UNTGZ.COM";
-#    1 while unlink "TS-UNTGZ.COM";
+    1 while unlink "TS-UNTGZ.COM";
 
     return ! $ret;
 }
@@ -925,20 +930,20 @@ sub _get_patches {
 
     my $ftp = Net::FTP->new($self->{pserver}, Debug => 0, Passive => 1) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't open '$self->{pserver}': $@";
+        Carp::carp( "[Net::FTP] Can't open '$self->{pserver}': $@" );
         return undef;
     };
 
     my @user_info = ( $self->{ftpusr}, $self->{ftppwd} );
     $ftp->login( @user_info ) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't login( @user_info )" ;
+        Carp::carp( "[Net::FTP] Can't login( @user_info )" );
         return undef;
     };
 
     $ftp->cwd( $self->{pdir} ) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't cd '$self->{pdir}'";
+        Carp::carp( "[Net::FTP] Can't cd '$self->{pdir}'" );
         return undef;
     };
 
@@ -993,7 +998,7 @@ sub _apply_patches {
     my $cwd = cwd();
     chdir $self->{ddir} or do {
         require Carp;
-        Carp::croak "Cannot chdir($self->{ddir}): $!";
+        Carp::croak( "Cannot chdir($self->{ddir}): $!" );
     };
 
     require Test::Smoke::Patcher;
@@ -1010,7 +1015,7 @@ sub _apply_patches {
         eval { $patcher->patch };
         if ( $@ ) {
              require Carp;
-	     Carp::carp "Error while patching:\n\t$@";
+	     Carp::carp( "Error while patching:\n\t$@" );
              next;
         }
 
@@ -1022,7 +1027,7 @@ sub _apply_patches {
     }
     chdir $cwd or do {
         require Carp;
-        Carp::croak "Cannot chdir($cwd) back: $!";
+        Carp::croak( "Cannot chdir($cwd) back: $!" );
     };
 }
 
@@ -1042,7 +1047,7 @@ sub _read_patch {
         require Compress::Zlib;
         my $unzip = Compress::Zlib::gzopen( $file, 'rb' ) or do {
             require Carp;
-            Carp::carp "Can't open '$file': $Compress::Zlib::gzerrno";
+            Carp::carp( "Can't open '$file': $Compress::Zlib::gzerrno" );
             return undef;
         };
 
@@ -1051,7 +1056,7 @@ sub _read_patch {
  
         unless ( $Compress::Zlib::gzerrno == Compress::Zlib::Z_STREAM_END() ) {
             require Carp;
-            Carp::carp "Error reading '$file': $Compress::Zlib::gzerrno" ;
+            Carp::carp( "Error reading '$file': $Compress::Zlib::gzerrno" );
         }
 
         $unzip->gzclose;
@@ -1155,6 +1160,11 @@ F<MANIFEST.SKIP>!).
 sub sync {
     my $self = shift;
 
+    $self->{cdir} eq $self->{ddir} and do {
+        require Carp;
+        Carp::croak( "Sourcetree cannot be copied onto it self!" );
+    };
+
     $self->pre_sync;
     require Test::Smoke::SourceTree;
 
@@ -1206,7 +1216,7 @@ sub new {
     my %args = @_;
     unless ( $args{hdir} ) {
         require Carp;
-        Carp::croak "No source-directory (hdir) specified for " . __PACKAGE__;
+        Carp::croak("No source-directory (hdir) specified for " . __PACKAGE__);
     }
     return bless \%args, $class;
 }
@@ -1426,15 +1436,15 @@ sub sync {
         my $cwd = Cwd::cwd();
         chdir $self->{fdir} or do {
             require Carp;
-            Carp::croak "Cannot chdir($self->{fdir}) in forest: $!";
+            Carp::croak( "Cannot chdir($self->{fdir}) in forest: $!" );
         };
         system( "$^X regen_headers.pl" ) == 0 or do {
             require Carp;
-            Carp::carp "Error while running 'regen_headers.pl'";
+            Carp::carp( "Error while running 'regen_headers.pl'" );
         };
         chdir $cwd or do {
             require Carp;
-            Carp::croak "Cannot chdir($cwd) back: $!";
+            Carp::croak( "Cannot chdir($cwd) back: $!" );
         };
     }
 
